type Query {
    # Returns a Product resource by ID.
    product(id: ID!): Product

    # Return a product by its handle.
    productByHandle(handle: String!): Product

    # Returns the product resource feedback for the currently authenticated app.
    productResourceFeedback(id: ID!): ProductResourceFeedback

    # Returns a list of the shop's product saved searches.
    productSavedSearches(
        after: String
        before: String
        first: Int
        last: Int
        reverse: Boolean = false
    ): SavedSearchConnection!

    # Returns a ProductVariant resource by ID.
    productVariant(id: ID!): ProductVariant

    # List of the product variants.
    productVariants(
        after: String
        before: String
        first: Int
        last: Int
        query: String
        reverse: Boolean = false
        savedSearchId: ID
        sortKey: ProductVariantSortKeys = ID
    ): ProductVariantConnection!

    # List of products.
    products(
        after: String
        before: String
        first: Int
        last: Int
        query: String
        reverse: Boolean = false
        savedSearchId: ID
        sortKey: ProductSortKeys = ID
    ): ProductConnection!
}

type Mutation {
    # Appends images to a product.
    productAppendImages(input: ProductAppendImagesInput!): ProductAppendImagesPayload

    # Changes the status of a product. This allows you to set the availability of the product across all channels.
    productChangeStatus(productId: ID! status: ProductStatus!): ProductChangeStatusPayload

    # Creates a product.
    #
    # If you need to create a product with many variants that are active at several locations,
    # especially with a lot of collections and tags, then you should first create the product with just the variants.
    #
    # After the product is created, you can activate the variants at locations
    # and add the other related objects to the product. This reduces the size of each mutation and increases the likelihood that it will
    # complete before the operation times out.
    #
    # The following example shows how you might break up product creation and object association into multiple steps:
    #
    # Create the product with variants. Don't specify any tags or collections on the product, and don't specify
    # inventory quantities for each variant.
    #
    # After the product is created, add tags to the product using the tagsAdd mutation, and add collections using the
    # collectionsAddProducts mutation.
    #
    #  Use the inventoryBulkToggleActivation mutation on each inventory item to activate it at the appropriate locations.
    #
    # After activating the variants at the locations, adjust inventory quantities at each location using the
    # inventoryBulkAdjustQuantityAtLocation mutation.
    productCreate(input: ProductInput! media: [CreateMediaInput!]): ProductCreatePayload

    # Creates media for a product
    productCreateMedia(productId: ID! media: [CreateMediaInput!]!): ProductCreateMediaPayload

    # Deletes a product, including all associated variants and media.
    #
    # As of API version 2023-01, if you need to delete a large product, such as one that has many variants
    # that are active at several locations,
    # you may encounter timeout errors. To avoid these timeout errors, you can instead use the asynchronous
    # ProductDeleteAsync mutation.
    productDelete(input: ProductDeleteInput!): ProductDeletePayload

    # Removes product images from the product.
    productDeleteImages(id: ID! imageIds: [ID!]!): ProductDeleteImagesPayload

    # Deletes media for a product.
    productDeleteMedia(productId: ID! mediaIds: [ID!]!): ProductDeleteMediaPayload

    # Duplicates a product.
    #
    # As of API version 2023-01, if you need to duplicate a large product, such as one that has many variants
    # that are active at several locations, you may encounter timeout errors.
    # To avoid these timeout errors, you can instead use the asynchronous ProductDuplicateAsync mutation.
    productDuplicate(
        productId: ID!
        newTitle: String!
        newStatus: ProductStatus
        includeImages: Boolean = false
    ): ProductDuplicatePayload

    # Updates an image of a product.
    productImageUpdate(productId: ID! image: ImageInput!): ProductImageUpdatePayload

    # Adds multiple selling plan groups to a product.
    productJoinSellingPlanGroups(id: ID! sellingPlanGroupIds: [ID!]!): ProductJoinSellingPlanGroupsPayload

    # Removes multiple groups from a product.
    productLeaveSellingPlanGroups(id: ID! sellingPlanGroupIds: [ID!]!): ProductLeaveSellingPlanGroupsPayload

    # Asynchronously reorders a set of images for a given product.
    productReorderImages(id: ID! moves: [MoveInput!]!): ProductReorderImagesPayload

    # Asynchronously reorders the media attached to a product.
    productReorderMedia(id: ID! moves: [MoveInput!]!): ProductReorderMediaPayload

    # Updates a product. If you update a product and only include some variants in the update,
    # then any variants not included will be deleted. To safely manage variants without the risk of
    # deleting excluded variants, use productVariantsBulkUpdate.
    # If you want to update a single variant, then use productVariantUpdate.
    productUpdate(input: ProductInput!): ProductUpdatePayload

    # Updates media for a product.
    productUpdateMedia(productId: ID! media: [UpdateMediaInput!]!): ProductUpdateMediaPayload

    # Appends media from a product to variants of the product.
    productVariantAppendMedia(productId: ID! variantMedia: [ProductVariantAppendMediaInput!]!): ProductVariantAppendMediaPayload

    # Creates a product variant.
    productVariantCreate(input: ProductVariantInput!): ProductVariantCreatePayload

    # Deletes a product variant.
    productVariantDelete(id: ID!): ProductVariantDeletePayload

    # Detaches media from product variants.
    productVariantDetachMedia(productId: ID! variantMedia: [ProductVariantDetachMediaInput!]!): ProductVariantDetachMediaPayload

    # Adds multiple selling plan groups to a product variant.
    productVariantJoinSellingPlanGroups(id: ID! sellingPlanGroupIds: [ID!]!): ProductVariantJoinSellingPlanGroupsPayload

    # Remove multiple groups from a product variant.
    productVariantLeaveSellingPlanGroups(id: ID! sellingPlanGroupIds: [ID!]!): ProductVariantLeaveSellingPlanGroupsPayload

    # Updates a product variant.
    productVariantUpdate(input: ProductVariantInput!): ProductVariantUpdatePayload

    # Creates product variants in bulk. If you want to create a single variant, then use productVariantCreate.
    productVariantsBulkCreate(variants: [ProductVariantsBulkInput!]! productId: ID!): ProductVariantsBulkCreatePayload

    # Deletes product variants in bulk. If you want to delete a single variant, then use productVariantDelete.
    productVariantsBulkDelete(variantsIds: [ID!]! productId: ID!): ProductVariantsBulkDeletePayload

    # Reorder product variants in bulk.
    productVariantsBulkReorder(productId: ID! positions: [ProductVariantPositionInput!]!): ProductVariantsBulkReorderPayload

    # Updates product variants in bulk. If you want to update a single variant, then use
    # productVariantUpdate.
    productVariantsBulkUpdate(variants: [ProductVariantsBulkInput!]! productId: ID!): ProductVariantsBulkUpdatePayload
}

# Specifies product images to append.
input ProductAppendImagesInput {
    # The ID of the product.
    id: ID!

    # A list of images to be appended to the product.
    images: [ImageInput!]!
}

# Specifies the input fields for an image.
input ImageInput {
    # A globally-unique identifier.
    id: ID

    # A word or phrase to share the nature or contents of an image.
    altText: String

    # The URL of the image. May be a staged upload URL.
    src: String
}

# Specifies the input fields required to create a product.
input ProductInput {
    # The description of the product, complete with HTML formatting.
    descriptionHtml: String


    # Whether a redirect is required after a new handle has been provided.
    # If true, then the old handle is redirected to the new one automatically.
    redirectNewHandle: Boolean

    # A unique, human-friendly string for the product.
    # Automatically generated from the product's title unless otherwise specified.
    handle: String

    # The SEO information associated with the product.
    seo: SEOInput

    # The product type specified by the merchant.
    productType: String

    # The standardized product type in the Shopify product taxonomy.
    standardizedProductType: StandardizedProductTypeInput

    # The product category in the Shopify product taxonomy.
    productCategory: ProductCategoryInput

    # The custom product type specified by the merchant.
    customProductType: String

    # A comma separated list of tags that have been added to the product.
    tags: [String!]

    # The theme template used when viewing the product in a store.
    templateSuffix: String

    # Whether the product is a gift card.
    giftCard: Boolean

    # The theme template used when viewing the gift card in a store.
    giftCardTemplateSuffix: String

    # The title of the product.
    title: String

    # The name of the product's vendor.
    vendor: String

    # The IDs of the collections that this product will be added to.
    collectionsToJoin: [ID!]

    # The IDs of collections that will no longer include the existing product.
    collectionsToLeave: [ID!]

    # Specifies the product to update in productUpdate or creates a new product if absent in productCreate.
    id: ID

    # The images to associate with the product.
    images: [ImageInput!]

    # The metafields to associate with this product.
    metafields: [MetafieldInput!]

    # The private metafields to associate with this product.
    privateMetafields: [PrivateMetafieldInput!]

    # List of custom product options (maximum of 3 per product).
    options: [String!]

    # A list of variants associated with the product.
    variants: [ProductVariantInput!]

    # The status of the product.
    status: ProductStatus

    # Whether the product can only be purchased with a selling plan (subscription).
    # Products that are sold exclusively on subscription can only be created on online stores.
    # If set to true on an already existing product, then the product will be marked unavailable on channels that don't support subscriptions.
    requiresSellingPlan: Boolean
}

# Specifies a product variant to create or update.
input ProductVariantInput {
    # The value of the barcode associated with the product.
    barcode: String

    # The compare-at price of the variant.
    compareAtPrice: Money

    # The Harmonized System code (or HS Tariff code) for the variant.
    harmonizedSystemCode: String

    # Specifies the product variant to update or create a new variant if absent.
    id: ID

    # The ID of the image that's associated with the variant.
    imageId: ID

    # The URL of an image to associate with the variant.
    # This field can only be used through mutations that create product images and must match one of the URLs being created on the product.
    imageSrc: String

    # The URL of the media to associate with the variant.
    # This field can only be used in mutations that create media images and must match one of the URLs being created on the product.
    # This field only accepts one value.
    mediaSrc: [String!]

    # Whether customers are allowed to place an order for the product variant when it's out of stock.
    inventoryPolicy: ProductVariantInventoryPolicy

    # The inventory quantities at each location where the variant is stocked.
    # Used as input only to the productVariantCreate mutation.
    inventoryQuantities: [InventoryLevelInput!]

    # The inventory item associated with the variant. Used for unit cost.
    inventoryItem: InventoryItemInput

    # Additional customizable information about the product variant.
    metafields: [MetafieldInput!]

    # The private metafields associated with the product.
    privateMetafields: [PrivateMetafieldInput!]

    # The custom properties that a shop owner uses to define product variants.
    options: [String!]

    # The order of the product variant in the list of product variants. The first position in the list is 1.
    position: Int

    # The price of the variant.
    price: Money

    # The product to create the variant for. Used as input only to the productVariantCreate mutation.
    productId: ID

    # Whether the variant requires shipping.
    requiresShipping: Boolean

    # The SKU for the variant.
    sku: String

    # Whether the variant is taxable.
    taxable: Boolean

    # The tax code associated with the variant.
    taxCode: String

    # The weight of the variant.
    weight: Float

    # The unit of weight that's used to measure the variant.
    weightUnit: WeightUnit
}

# Specifies the input fields for an inventory level.
input InventoryLevelInput {
    # The available quantity of an inventory item at a location.
    availableQuantity: Int!

    # The ID of a location.
    locationId: ID!
}

# Specifies the input fields for an inventory item.
input InventoryItemInput {
    # Unit cost associated with the inventory item, the currency is the shop's default currency.
    cost: Decimal

    # Whether the inventory item is tracked.
    tracked: Boolean
}

# Return type for productAppendImages mutation.
type ProductAppendImagesPayload {
    # List of new images appended to the product.
    newImages: [Image!]

    # The product object.
    product: Product

    # The list of errors that occurred from executing the mutation.
    userErrors: [UserError!]!
}

# Return type for productChangeStatus mutation.
type ProductChangeStatusPayload {
    # The product object.
    product: Product

    # The list of errors that occurred from executing the mutation.
    userErrors: [ProductChangeStatusUserError!]!
}

# Provides the fields and values to use when adding a standard product type to a product.
# The Shopify product taxonomy contains the full list of available values.
input StandardizedProductTypeInput {
    # The id of the node in the Shopify taxonomy that represents the product type.
    productTaxonomyNodeId: ID!
}

# The input fields to use when adding a product category to a product.
# The Shopify product taxonomy contains the full list of available values.
input ProductCategoryInput {
    # The ID of the node in the Shopify taxonomy that represents the product category.
    productTaxonomyNodeId: ID!
}

# Specifies the input fields required to create a media object.
input CreateMediaInput {
    # The original source of the media object. This might be an external URL or a staged upload URL.
    originalSource: String!

    # The alt text associated with the media.
    alt: String

    # The media content type.
    mediaContentType: MediaContentType!
}

# A single move of an object to a specific position in a set, using a zero-based index.
input MoveInput {
    # The ID of the object to be moved.
    id: ID!

    # The new position of the object in the set.
    newPosition: UnsignedInt64!
}

# Specifies the product to delete.
input ProductDeleteInput {
    # The ID of the product.
    id: ID!
}

# Specifies the input fields required to update a media object.
input UpdateMediaInput {
    # Specifies the media to update.
    id: ID!

    # The source from which to update the media preview image. May be an external URL or staged upload URL.
    previewImageSource: String

    # The alt text associated to the media.
    alt: String
}

# Specifies the input fields required to append media to a single variant.
input ProductVariantAppendMediaInput {
    # Specifies the variant to which media will be appended.
    variantId: ID!

    # Specifies the media to append to the variant.
    mediaIds: [ID!]!
}

# Specifies the input fields required to detach media from a single variant.
input ProductVariantDetachMediaInput {
    # Specifies the variant from which media will be detached.
    variantId: ID!

    # Specifies the media to detach from the variant.
    mediaIds: [ID!]!
}

# Specifies a product variant to create as part of a variant bulk mutation.
input ProductVariantsBulkInput {
    # The value of the barcode associated with the product variant.
    barcode: String

    # The compare-at price of the variant.
    compareAtPrice: Money

    # The Harmonized System code (or HS Tariff code) for the variant.
    harmonizedSystemCode: String

    # Specifies the product variant to update or delete.
    id: ID

    # The ID of the image that's associated with the variant.
    imageId: ID

    # The URL of an image to associate with the variant. This field can only be used through mutations that create product images and must match one of the URLs being created on the product.
    imageSrc: String

    # The URL of the media to associate with the variant.
    mediaSrc: [String!]

    # Whether customers are allowed to place an order for the variant when it's out of stock.
    inventoryPolicy: ProductVariantInventoryPolicy

    # The inventory quantities at each location where the variant is stocked. The number of elements
    # in the array of inventory quantities cannot exceed 10 and the amount specified for the plan.
    # Used as input only to the productVariantCreate mutation.
    inventoryQuantities: [InventoryLevelInput!]

    # The inventory item associated with the variant, used for unit cost.
    inventoryItem: InventoryItemInput

    # The additional customizable information about the product variant.
    metafields: [MetafieldInput!]

    # The private metafields associated with the product.
    privateMetafields: [PrivateMetafieldInput!]

    # The custom properties that a shop owner uses to define product variants.
    options: [String!]

    # The price of the variant.
    price: Money

    # Whether the variant requires shipping.
    requiresShipping: Boolean

    # The SKU for the variant.
    sku: String

    # Whether the variant is taxable.
    taxable: Boolean

    # The tax code associated with the variant.
    taxCode: String

    # The weight of the variant.
    weight: Float

    # The unit of weight that's used to measure the variant.
    weightUnit: WeightUnit
}

# Represents a product variant position.
input ProductVariantPositionInput {
    # Specifies the ID of the product variant to update.
    id: ID!

    # The order of the product variant in the list of product variants. The first position in the list is 1.
    position: Int!
}

# Return type for productCreate mutation.
type ProductCreatePayload {
    # The product object.
    product: Product

    # The shop associated with the product.
    shop: Shop!

    # The list of errors that occurred from executing the mutation.
    userErrors: [UserError!]!
}

# Return type for productCreateMedia mutation.
type ProductCreateMediaPayload {
    # The newly created media.
    media: [Media!]

    # The list of errors that occurred from executing the mutation.
    mediaUserErrors: [MediaUserError!]!

    # The product associated with the media.
    product: Product
}

# Return type for productDelete mutation.
type ProductDeletePayload {
    # The ID of the deleted product.
    deletedProductId: ID

    # The shop associated with the product.
    shop: Shop!

    # The list of errors that occurred from executing the mutation.
    userErrors: [UserError!]!
}

# Return type for productImageUpdate mutation.
type ProductImageUpdatePayload {
    # The image that has been updated.
    image: Image

    # The list of errors that occurred from executing the mutation.
    userErrors: [UserError!]!
}

# Return type for productDeleteImages mutation.
type ProductDeleteImagesPayload {
    # The array of image IDs to delete.
    deletedImageIds: [ID!]!

    # The product object.
    product: Product

    # The list of errors that occurred from executing the mutation.
    userErrors: [UserError!]!
}

# Return type for productDeleteMedia mutation.
type ProductDeleteMediaPayload {
    # List of media IDs which were deleted.
    deletedMediaIds: [ID!]

    # List of product image IDs which were deleted.
    deletedProductImageIds: [ID!]

    # The list of errors that occurred from executing the mutation.
    mediaUserErrors: [MediaUserError!]!

    # The product associated with the deleted media.
    product: Product
}

# Return type for productDuplicate mutation.
type ProductDuplicatePayload {
    # The asynchronous job that duplicates the product images.
    imageJob: Job

    # The duplicated product.
    newProduct: Product

    # The user's shop.
    shop: Shop!

    # The list of errors that occurred from executing the mutation.
    userErrors: [UserError!]!
}

# Return type for productJoinSellingPlanGroups mutation.
type ProductJoinSellingPlanGroupsPayload {
    # The product object.
    product: Product

    # The list of errors that occurred from executing the mutation.
    userErrors: [SellingPlanGroupUserError!]!
}

# Return type for productLeaveSellingPlanGroups mutation.
type ProductLeaveSellingPlanGroupsPayload {
    # The product object.
    product: Product

    # The list of errors that occurred from executing the mutation.
    userErrors: [SellingPlanGroupUserError!]!
}

# Return type for productReorderImages mutation.
type ProductReorderImagesPayload {
    # The asynchronous job which reorders the images.
    job: Job

    # The list of errors that occurred from executing the mutation.
    userErrors: [UserError!]!
}

# Return type for productReorderMedia mutation.
type ProductReorderMediaPayload {
    # The asynchronous job which reorders the media.
    job: Job

    # The list of errors that occurred from executing the mutation.
    mediaUserErrors: [MediaUserError!]!
}

# Return type for productUpdate mutation.
type ProductUpdatePayload {
    # The updated product object.
    product: Product

    # The list of errors that occurred from executing the mutation.
    userErrors: [UserError!]!
}

# Return type for productUpdateMedia mutation.
type ProductUpdateMediaPayload {
    # The updated media object.
    media: [Media!]

    # The list of errors that occurred from executing the mutation.
    mediaUserErrors: [MediaUserError!]!

    # The product on which media was updated.
    product: Product
}

# Return type for productVariantAppendMedia mutation.
type ProductVariantAppendMediaPayload {
    # The product associated with the variants and media.
    product: Product

    # The product variants that were updated.
    productVariants: [ProductVariant!]

    # The list of errors that occurred from executing the mutation.
    userErrors: [MediaUserError!]!
}

# Return type for productVariantCreate mutation.
type ProductVariantCreatePayload {
    # The product associated with the variant.
    product: Product

    # The successfully created variant.
    productVariant: ProductVariant

    # The list of errors that occurred from executing the mutation.
    userErrors: [UserError!]!
}

# Return type for productVariantDelete mutation.
type ProductVariantDeletePayload {
    # The ID of the deleted product variant.
    deletedProductVariantId: ID

    # The product associated with the deleted product variant.
    product: Product

    # The list of errors that occurred from executing the mutation.
    userErrors: [UserError!]!
}

# Return type for productVariantDetachMedia mutation.
type ProductVariantDetachMediaPayload {
    # The product associated with the variants and media.
    product: Product

    # The product variants that were updated.
    productVariants: [ProductVariant!]

    # The list of errors that occurred from executing the mutation.
    userErrors: [MediaUserError!]!
}

# Return type for productVariantJoinSellingPlanGroups mutation.
type ProductVariantJoinSellingPlanGroupsPayload {
    # The product variant object.
    productVariant: ProductVariant

    # The list of errors that occurred from executing the mutation.
    userErrors: [SellingPlanGroupUserError!]!
}

# Return type for productVariantLeaveSellingPlanGroups mutation.
type ProductVariantLeaveSellingPlanGroupsPayload {
    # The product variant object.
    productVariant: ProductVariant

    # The list of errors that occurred from executing the mutation.
    userErrors: [SellingPlanGroupUserError!]!
}

# Return type for productVariantUpdate mutation.
type ProductVariantUpdatePayload {
    # The product associated with the variant.
    product: Product

    # The updated variant.
    productVariant: ProductVariant

    # The list of errors that occurred from executing the mutation.
    userErrors: [UserError!]!
}

# Return type for productVariantsBulkCreate mutation.
type ProductVariantsBulkCreatePayload {
    # The updated product object.
    product: Product

    # The newly created variants.
    productVariants: [ProductVariant!]

    # The list of errors that occurred from executing the mutation.
    userErrors: [ProductVariantsBulkCreateUserError!]!
}

# Return type for productVariantsBulkDelete mutation.
type ProductVariantsBulkDeletePayload {
    # The updated product object.
    product: Product

    # The list of errors that occurred from executing the mutation.
    userErrors: [ProductVariantsBulkDeleteUserError!]!
}

# Return type for productVariantsBulkReorder mutation.
type ProductVariantsBulkReorderPayload {
    # The updated product.
    product: Product

    # The list of errors that occurred from executing the mutation.
    userErrors: [ProductVariantsBulkReorderUserError!]!
}

# Return type for productVariantsBulkUpdate mutation.
type ProductVariantsBulkUpdatePayload {
    # The updated product object.
    product: Product

    # The updated variants.
    productVariants: [ProductVariant!]

    # The list of errors that occurred from executing the mutation.
    userErrors: [ProductVariantsBulkUpdateUserError!]!
}

# Reports the status of product for a Sales Channel or Storefront API.
# This might include why a product is not available in a Sales Channel
# and how a merchant might fix this.
type ProductResourceFeedback {
    # The time when the feedback was generated. Used to help determine whether
    # incoming feedback is outdated compared to existing feedback.
    feedbackGeneratedAt: DateTime!

    # The feedback messages presented to the merchant.
    messages: [String!]!

    # The ID of the product associated with the feedback.
    productId: ID!

    # The timestamp of the product associated with the feedback.
    productUpdatedAt: DateTime!

    # Conveys the state of the feedback and whether it requires merchant action or not.
    state: ResourceFeedbackState!
}
